var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        if (typeof b !== "function" && b !== null)
            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
import * as elements from '@yellicode/elements';
import { CodeWriter, CodeWriterUtility } from '@yellicode/core';
import { JavaTypeNameProvider } from './java-type-name-provider';
import { JavaCommentWriter } from './comment-writer';
import { XmlDocUtility } from './xml-doc-utility';
import { DefinitionBuilder } from './definition-builder';
/**
 * A CodeWriter for writing Java code from code generation templates. This writer can write classes, interfaces, structs and enumerations and also
 * contains functions for writing namespace blocks and using directives. The JavaWriter is compatible with Yellicode models but can also work
 * independently.
 */
var JavaWriter = /** @class */ (function (_super) {
    __extends(JavaWriter, _super);
    /**
     * Constructor. Creates a new JavaWriter instance using the TextWriter and options provided.
     * @param writer The template's current TextWriter.
     * @param options Optional: the global options for this writer.
     */
    function JavaWriter(writer, options) {
        var _this = _super.call(this, writer) || this;
        if (!options)
            options = {};
        _this.typeNameProvider =
            options.typeNameProvider || new JavaTypeNameProvider();
        _this.definitionBuilder = new DefinitionBuilder(_this.typeNameProvider);
        _this.indentString = '  ';
        _this.maxCommentWidth = options.maxCommentWidth || 100;
        _this.commentWriter = new JavaCommentWriter(writer, _this.maxCommentWidth);
        return _this;
    }
    JavaWriter.prototype.writeFileContent = function (definition, contents) {
        this.writeLine("package " + definition.package + ";");
        contents(this);
        return this;
    };
    JavaWriter.prototype.writeMainMethod = function (contents) {
        this.writeLine('public static void main(String[] args) {');
        if (contents) {
            this.increaseIndent();
            contents(this);
            this.decreaseIndent();
        }
        this.writeLine('}');
        return this;
    };
    /**
     * Writes 1 or more using directives, each on a new line.
     * @param values A collection of strings, typically namespace names.
     */
    JavaWriter.prototype.writeUsingDirectives = function () {
        var _this = this;
        var values = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            values[_i] = arguments[_i];
        }
        values.forEach(function (v) {
            _this.writeLine("using " + v + ";");
        });
        return this;
    };
    /**
     * Writes an indented block of code, wrapped in opening and closing brackets.
     * @param contents A callback function that writes the contents.
     */
    JavaWriter.prototype.writeCodeBlock = function (contents) {
        this.write(' {');
        this.writeEndOfLine();
        this.increaseIndent();
        if (contents)
            contents(this);
        this.decreaseIndent();
        this.writeLine('}');
        return this;
    };
    JavaWriter.prototype.writeNamespaceBlock = function (data, contents, options) {
        if (!data)
            return this;
        var definition = elements.isPackage(data)
            ? this.definitionBuilder.buildPackageDefinition(data, options)
            : data;
        this.writeLine("namespace " + definition.name);
        this.writeCodeBlock(contents);
        return this;
    };
    JavaWriter.prototype.writeClassBlock = function (data, contents, options) {
        if (!data)
            return this;
        var definition = elements.isType(data)
            ? this.definitionBuilder.buildClassDefinition(data, options)
            : data;
        if (definition.docComment) {
            this.writeDocComment(definition.docComment);
        }
        this.writeIndent();
        this.writeAccessModifier(definition);
        if (definition.isStatic) {
            this.write('static ');
        }
        else if (definition.isAbstract) {
            // note that static classes cannot be abstract, hence the else-statement
            this.write('abstract ');
        }
        if (definition.isPublic) {
            this.write('public ');
        }
        this.write("class " + definition.name);
        var hasGeneralizations = false;
        hasGeneralizations = this.writeExtends(definition.extends);
        this.writeImplements(hasGeneralizations, definition.implements);
        this.writeCodeBlock(contents);
        return this;
    };
    JavaWriter.prototype.writeStructBlock = function (data, contents, options) {
        if (!data)
            return this;
        var definition = elements.isType(data)
            ? this.definitionBuilder.buildStructDefinition(data, options)
            : data;
        if (definition.docComment) {
            this.writeDocComment(definition.docComment);
        }
        this.writeIndent();
        this.writeAccessModifier(definition);
        if (definition.isPublic) {
            this.write('public ');
        }
        this.write("struct " + definition.name);
        this.writeImplements(false, definition.implements);
        this.writeEndOfLine();
        this.writeCodeBlock(contents);
        return this;
    };
    JavaWriter.prototype.writeInterfaceBlock = function (data, contents, options) {
        if (!data)
            return this;
        var definition = elements.isType(data)
            ? this.definitionBuilder.buildInterfaceDefinition(data, options)
            : data;
        if (definition.docComment) {
            this.writeDocComment(definition.docComment);
        }
        this.writeIndent();
        this.writeAccessModifier(definition);
        if (definition.isPublic) {
            this.write('public ');
        }
        this.write("interface " + definition.name);
        this.writeExtends(definition.inherits);
        this.writeEndOfLine();
        this.writeCodeBlock(contents);
        return this;
    };
    JavaWriter.prototype.writeEnumerationBlock = function (data, contents, options) {
        if (!data)
            return this;
        var definition = elements.isType(data)
            ? this.definitionBuilder.buildEnumDefinition(data, options)
            : data;
        if (definition.docComment) {
            this.writeDocComment(definition.docComment);
        }
        this.writeIndent();
        this.writeAccessModifier(definition);
        this.write("enum " + definition.name);
        this.writeEndOfLine();
        this.writeCodeBlock(contents);
        return this;
    };
    JavaWriter.prototype.writeEnumeration = function (data, options) {
        var _this = this;
        if (!data)
            return this;
        var definition = elements.isType(data)
            ? this.definitionBuilder.buildEnumDefinition(data, options)
            : data;
        this.writeEnumerationBlock(definition, function () {
            if (definition.members) {
                definition.members.forEach(function (memberDefinition) {
                    _this.writeEnumMember(memberDefinition);
                });
            }
        });
        return this;
    };
    JavaWriter.prototype.writeEnumMember = function (data, options, isLast) {
        if (!data)
            return this;
        var definition = elements.isEnumerationLiteral(data)
            ? this.definitionBuilder.buildEnumMemberDefinition(data, isLast, options)
            : data;
        if (definition.docComment) {
            this.writeDocComment(definition.docComment);
        }
        this.writeIndent();
        this.write(definition.name);
        if (definition.value != null) {
            // using '!=' on purpose
            this.write(" = " + definition.value);
        }
        if (!definition.isLast) {
            this.write(',');
        }
        this.writeEndOfLine();
        return this;
    };
    JavaWriter.prototype.writeMethodDeclaration = function (data, options) {
        if (!data)
            return this;
        var definition = elements.isOperation(data)
            ? this.definitionBuilder.buildMethodDefinition(data, options)
            : data;
        if (definition.docComment) {
            this.writeDocComment(definition.docComment);
        }
        if (definition.parameters) {
            this.writeJavaDocParagraph(XmlDocUtility.getXmlDocLinesForInOutParameters(definition.parameters));
        }
        var xmlDocReturns = XmlDocUtility.getXmlDocLineForReturnParameter(definition);
        if (xmlDocReturns) {
            this.writeJavaDocParagraph([xmlDocReturns]);
        }
        this.writeIndent();
        if (!definition.isPublic) {
            this.writeAccessModifier(definition); // Partial methods are implicitly private
        }
        if (definition.isPublic) {
            this.write('public ');
        }
        if (definition.isStatic) {
            this.write('static ');
        }
        else if (definition.isAbstract) {
            this.write('abstract ');
        }
        // partial methods must return void, intentional trailing white space
        else
            this.write((definition.returnTypeName || 'void') + " "); // intentional trailing white space
        this.write(definition.name + "(");
        if (definition.parameters) {
            this.writeParameters(definition.parameters);
        }
        this.write(');');
        this.writeEndOfLine();
        return this;
    };
    JavaWriter.prototype.writeMethodBlock = function (data, contents, options) {
        if (!data)
            return this;
        var definition = elements.isOperation(data)
            ? this.definitionBuilder.buildMethodDefinition(data, options)
            : data;
        // Write the documentation
        if (definition.docComment) {
            this.writeDocComment(definition.docComment);
        }
        if (definition.parameters) {
            this.writeJavaDocParagraph(XmlDocUtility.getXmlDocLinesForInOutParameters(definition.parameters));
        }
        var xmlDocReturns = definition.isConstructor
            ? null
            : XmlDocUtility.getXmlDocLineForReturnParameter(definition);
        if (xmlDocReturns) {
            this.writeJavaDocParagraph([xmlDocReturns]);
        }
        // Start of the actual method
        this.writeIndent();
        // Write the access modifier
        if (definition.isPublic) {
            this.write('public ');
        }
        else {
            this.writeAccessModifier(definition);
        }
        // Write the non-access modifier
        this.writeNonAccessModifier(definition.nonAccessModifier);
        // Write the return type
        this.write((definition.returnTypeName || 'void') + " "); // intentional trailing white space
        this.write(definition.name + "(");
        if (definition.parameters) {
            this.writeParameters(definition.parameters);
        }
        this.write(')');
        if (definition.isAbstract) {
            this.writeEndOfLine(';');
        }
        else {
            this.writeCodeBlock(contents);
        }
        return this;
    };
    /**
     * Writes a setter
     * @param param
     * @param methodName Method name without the set prefix
     */
    JavaWriter.prototype.writeSetterMethod = function (param, methodName) {
        if (!methodName) {
            methodName = param.name.replace(/^.{1}/, function (letter) {
                return letter.toLocaleUpperCase();
            });
        }
        this.writeLine("public void set" + methodName + "(" + param.typeName + " " + param.name + ") {");
        this.increaseIndent();
        this.writeLine("this." + param.name + " = " + param.name + ";");
        this.decreaseIndent();
        this.writeLine("}");
    };
    /**
     * Writes a setter
     * @param param
     * @param methodName Method name without the set prefix
     */
    JavaWriter.prototype.writeGetterMethod = function (param, methodName) {
        if (!methodName) {
            methodName = param.name.replace(/^.{1}/, function (letter) {
                return letter.toLocaleUpperCase();
            });
        }
        this.writeLine("public " + param.typeName + " get" + methodName + "() {");
        this.increaseIndent();
        this.writeLine("return this." + param.name + ";");
        this.decreaseIndent();
        this.writeLine("}");
    };
    JavaWriter.prototype.writeParameters = function (data, options) {
        var _this = this;
        if (!data || !data.length)
            // without at least 1 element, we cannot determine the type
            return this;
        var definitions = elements.isParameter(data[0])
            ? this.definitionBuilder.buildParameterDefinitions(data, options)
            : data;
        if (!definitions.length)
            return this;
        var i = 0;
        definitions.forEach(function (p) {
            if (i > 0) {
                _this.write(', ');
            }
            _this.write(p.typeName);
            if (p.isNullable) {
                _this.write('?');
            }
            _this.write(" " + p.name);
            if (p.defaultValue) {
                _this.write(" = " + p.defaultValue);
            }
            i++;
        });
        return this;
    };
    JavaWriter.prototype.writeProperty = function (prop) {
        if (prop.docComment) {
            this.writeDocComment(prop.docComment);
        }
        // Start a new, indented line
        this.writeIndent();
        // Access modifier
        this.writeAccessModifier(prop);
        // Static
        if (prop.isStatic)
            this.write('static ');
        // Type
        this.write(prop.typeName + " ");
        // Name
        this.write(prop.name);
        // Default value
        this.writePropertyValue(prop);
        // End
        this.write(';');
        this.writeEndOfLine();
        return this;
    };
    JavaWriter.prototype.writePropertyValue = function (prop) {
        var value = prop.defaultValue;
        if (value === undefined)
            return this;
        if (prop.typeName == 'String') {
            this.write(" = \"" + value + "\"");
        }
        else {
            this.write(" = " + value);
        }
        return this;
    };
    JavaWriter.prototype.writeAccessModifier = function (visibilityKind) {
        if (!visibilityKind)
            return this;
        var accessModifier = typeof visibilityKind == 'number'
            ? DefinitionBuilder.getAccessModifierString(visibilityKind) // VisibilityKind
            : visibilityKind.accessModifier; // TypeDefinition
        if (!accessModifier)
            return this;
        this.write(accessModifier);
        this.writeWhiteSpace();
        return this;
    };
    JavaWriter.prototype.writeNonAccessModifier = function (nonAccessModifier) {
        if (!nonAccessModifier)
            return this;
        this.write(nonAccessModifier);
        this.writeWhiteSpace();
        return this;
    };
    JavaWriter.prototype.getTypeName = function (element) {
        if (!element)
            return null;
        return this.typeNameProvider.getTypeName(element);
    };
    // #region deprecated
    /**
     * Writes a method declaration without a body.
     * @deprecated Use the writeMethodDeclaration() function instead.
     */
    JavaWriter.prototype.writeInterfaceMethod = function (operation, options) {
        console.warn('writeInterfaceMethod is deprecated. Use the writeMethodDeclaration() function instead. ');
        this.writeMethodDeclaration(operation, options);
        return this;
    };
    /**
     * Writes an indented block of code, wrapped in a method declaration and opening and closing brackets.
     * @deprecated Use the writeMethodBlock() function instead.
     */
    JavaWriter.prototype.writeClassMethodBlock = function (operation, contents, options) {
        console.warn('writeClassMethodBlock is deprecated. Use the writeMethodBlock() function instead. ');
        this.writeMethodBlock(operation, contents, options);
        return this;
    };
    JavaWriter.prototype.writeDocComment = function (data) {
        if (elements.isNamedElement(data)) {
            // we have no isElement check, but this will do
            data = DefinitionBuilder.buildXmlDocSummary(data);
        }
        if (!data)
            return this;
        var array = [];
        if (typeof data == 'string') {
            // string
            array.push(data);
        }
        else {
            // string[]
            array.push.apply(array, data);
        }
        if (!array.length)
            return this;
        this.writeLine('/**');
        this.commentWriter.writeCommentLines(array, ' * ');
        this.writeLine(' */');
        return this;
    };
    JavaWriter.prototype.writeJavaDocParagraph = function (data) {
        if (data == null)
            return this;
        var lines;
        if (typeof data == 'string') {
            lines = [data];
        }
        else
            lines = data;
        this.writeJsDocLines(lines);
        return this;
    };
    JavaWriter.prototype.writeJsDocLines = function (lines) {
        var _this = this;
        if (lines.length === 0)
            return this;
        this.writeLine('/**');
        // Split here
        lines.forEach(function (line) {
            var lineLength = line ? line.length : 0;
            if (_this.maxCommentWidth > 0 && lineLength > _this.maxCommentWidth) {
                // See if we can split the line
                var split = CodeWriterUtility.wordWrap(line, _this.maxCommentWidth);
                split.forEach(function (s) {
                    _this.writeLine("* " + s);
                });
            }
            else
                _this.writeLine("* " + line);
        });
        this.writeLine('*/');
        return this;
    };
    JavaWriter.prototype.writeDelimitedCommentParagraph = function (data) {
        if (typeof data == 'string') {
            this.commentWriter.writeDelimitedCommentParagraph(data);
        }
        else
            this.commentWriter.writeDelimitedCommentLines(data);
        return this;
    };
    JavaWriter.prototype.writeDelimitedCommentLines = function (paragraph) {
        console.warn('writeDelimitedCommentLines is deprecated. Use the writeDelimitedCommentParagraph() function instead.');
        this.commentWriter.writeDelimitedCommentLines(paragraph);
        return this;
    };
    //#endregion Xml Docs
    JavaWriter.prototype.joinWrite = function (collection, separator, getStringFunc) {
        var _this = this;
        var isFirst = true;
        collection.forEach(function (c) {
            var value = getStringFunc(c);
            if (!value)
                return;
            if (isFirst) {
                isFirst = false;
            }
            else
                _this.write(separator);
            _this.write(value);
        });
    };
    JavaWriter.prototype.writeExtends = function (ext) {
        if (!ext || !ext.length)
            return false;
        this.write(' extends ');
        this.joinWrite(ext, ', ', function (name) { return name; });
        return true;
    };
    JavaWriter.prototype.writeImplements = function (hasGeneralizations, impl) {
        if (!impl || !impl.length)
            return false;
        this.write(hasGeneralizations ? ', ' : ' : ');
        this.joinWrite(impl, ', ', function (name) { return name; });
        return true;
    };
    /**
     *
     * @param classList The class list to imported
     */
    JavaWriter.prototype.writeImports = function (classList) {
        var _this = this;
        classList.forEach(function (cls) {
            _this.writeLine(cls ? "import " + cls + ";" : '');
        });
    };
    return JavaWriter;
}(CodeWriter));
export { JavaWriter };
